package com.school.project.controller;

import com.school.project.dto.CourseDTO;
import com.school.project.mapper.CourseMapper;
import com.school.project.model.Course;
import com.school.project.service.implementation.CourseServiceImp;
import com.school.project.service.interfaces.CourseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.sql.SQLIntegrityConstraintViolationException;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Optional;

@Controller
@RequestMapping()
public class CourseController {
    @Autowired(required = true)
    public CourseServiceImp courseServiceImpl;

    @PostMapping("/courses")
    public ResponseEntity<CourseDTO> saveCourse(@RequestBody CourseDTO courseDTO) {

        try {
            CourseDTO course = courseServiceImpl.saveCourse(courseDTO);
            return ResponseEntity.status(HttpStatus.OK).body(course);
        } catch (Exception e) {

            return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();
        }
    }

    @GetMapping("/courses")
    public ResponseEntity<List<CourseDTO>> readAllCourses() {
        List<CourseDTO> course = courseServiceImpl.readAllCourse();

        if (course.size() <= 0) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        } else {
            return ResponseEntity.status(HttpStatus.OK).body(course);
        }
    }

    @GetMapping("/courses/{id}")
    public ResponseEntity<CourseDTO> readCourseById(@PathVariable(name = "id") Long id) {

        try {
            CourseDTO course = courseServiceImpl.readCourseById(id);
            return ResponseEntity.status(HttpStatus.OK).body(course);
        } catch (NoSuchElementException e) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        }

    }

    @PutMapping("/courses/{id}")
    public ResponseEntity<CourseDTO> updateCourseById(@RequestBody CourseDTO courseDTO, @PathVariable(name = "id") Long id) {
        try {
            CourseDTO updateCourse = courseServiceImpl.updateCourseById(courseDTO, id);
            return ResponseEntity.status(HttpStatus.OK).body(updateCourse);

        } catch (NoSuchElementException e) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        }

    }

    @DeleteMapping("/courses/{id}")
    public ResponseEntity deleteCourseById(@PathVariable(name = "id") Long id) {

        try {
            courseServiceImpl.deleteCourseById(id);
            return ResponseEntity.status(HttpStatus.NO_CONTENT).build();
        } catch (EmptyResultDataAccessException e) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        }

    }
}
