package com.school.project.controller;

import com.school.project.dto.CourseDTO;
import com.school.project.dto.DepartmentDTO;
import com.school.project.service.implementation.DepartmentServiceImp;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.sql.SQLIntegrityConstraintViolationException;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Optional;

@RestController
public class DepartmentController {
    @Autowired(required = true)
    public DepartmentServiceImp departmentServiceImp;

    @GetMapping("departments")
    public ResponseEntity<List<DepartmentDTO>> readAllDepartments() {
        List<DepartmentDTO> departmentDTO = departmentServiceImp.readDepartment();

        if (departmentDTO.isEmpty()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        } else {
            return ResponseEntity.of(Optional.of(departmentDTO));
        }
    }

    @GetMapping("departmentId/{id}")
    public ResponseEntity<DepartmentDTO> readDepartmentById(@PathVariable("id") Long id) {
        try {
            DepartmentDTO departmentDTO = departmentServiceImp.readDepartmentById(id);

            return ResponseEntity.of(Optional.of(departmentDTO));

        } catch (NoSuchElementException e) {

            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        }

    }

    @GetMapping("departmentName/{name}")
    public ResponseEntity<DepartmentDTO> findDepartmentByName(@PathVariable("name") String name) {
        DepartmentDTO departmentDTO = departmentServiceImp.findDepartmentByName(name);

        if (departmentDTO == null) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        } else {
            return ResponseEntity.of(Optional.of(departmentDTO));
        }
    }

    @PostMapping("departments")
    public ResponseEntity<DepartmentDTO> saveDepartment(@RequestBody DepartmentDTO department) {

        try {
            DepartmentDTO departmentDTO = departmentServiceImp.saveDepartment(department);

            return ResponseEntity.status(HttpStatus.OK).body(departmentDTO);

        } catch (Exception exception) {

            return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();

        }

    }

    @PutMapping("departments/{id}")
    public ResponseEntity<DepartmentDTO> updateDepartmentById(@RequestBody DepartmentDTO department, @PathVariable("id") Long id) {
        try {
            DepartmentDTO updateDepartment = departmentServiceImp.updateDepartment(department, id);
            return ResponseEntity.status(HttpStatus.OK).body(updateDepartment);
        } catch (NoSuchElementException e) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        }
    }

    @DeleteMapping("departments/{id}")
    public ResponseEntity<Object> deleteDepartmentById(@PathVariable("id") Long id) {
        try {
            departmentServiceImp.deleteDepartmentById(id);
            return ResponseEntity.status(HttpStatus.NO_CONTENT).build();
        } catch (EmptyResultDataAccessException e) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        }
    }
}
